data intoit;
input stimulus _1-_9;
* Suppose you have 17 items that were rated using a 1-9 point
favorability scale.  You need to construct a data set with one
observation for each item.  Each observation will have a numeric
variable named "STIMULUS" and nine numeric category rating 
variables named "_1" through "_9".  The category rating variables
will contain the number of times that subjects rated
the favorability of the item using a given response category,
where 1=Very Unfavorable and 9=Very Favorable.;
cards;
   1   1  3  1  3 10 21 28 15  8
   3   0  1  5  7 24 32 11  7  3
   6   1  2  3  8 23 16 27  7  3
   8  28 24 20 13  3  1  0  0  1
   9   3  5  5 18 43 10  4  2  0
  10   0  1  1  0  3 16 32 24 13
  13  28 27 14 13  5  0  0  3  0
  14  51 20 15  2  1  0  0  1  0
  15  25 24 20 16  5  0  0  0  0
  16  58 21  5  1  1  0  0  2  2
  17   1  0  1  0  0  2 10 22 54
  18   0  2  8 10 38 20  9  2  1
  19  70 13  5  0  1  1  0  0  0
  21  53 24  8  5  0  0  0  0  0
  22   2  1  0  0 85  1  0  1  0
  23   1  3  1  8 10 28 22 14  3
  24   1  0  1  0  3  6 14 29 36
proc print;
run;



%macro catjug(dsn,lbp,ubp,ncats,catlabs,outdsn1,outdsn2);

*********************************************************


 CCCC    A    TTTTT  JJJJJ  U   U   GGGG
C       A A     T      J    U   U  G
C      AAAAA    T      J    U   U  G
C      A   A    T      J    U   U  G GGG
C      A   A    T    J J    U   U  G   G
 CCCC  A   A    T    JJJ     UUU    GGGG


PROGRAMMER: JAMES S. ROBERTS
DATE:       5/7/98
This macro has been tested on a variety of data sets.  However, the
author assumes no responsibility for the accuracy of the results.


Macro CATJUG calculates Thurstone scale values, scale value standard devia-
tions, and category boundaries using a variant of the Thurstone successive
intervals procedure that is described in Torgerson (1958).  The algebraic
solution for complete data is given in Table 7 on page 226 of Torgerson.
The solution for incomplete data is given in Table 9 of Torgerson.

Parameters input to CATJUG are:

DSN - name of the SAS data set containing the stimulus ID values and the
      category rating frequencies

LBP - the lower bound for the proportion of category judgments that
      will be considered.  Proportions below this bound are set to
      missing

UBP - the upper bound for the proportion of category judgments that
      will be considered.  Proportions above this bound are set to
      missing

NCATS - the number of points on the category rating scale (i.e., the
        number of rating categories)

CATLABS - the numeric portion of the SAS variable names corresponding
          to the variables which contain the category rating
          frequencies.  The variables must be named _n, where n is an
          integer value.

OUTDSN1 - the name of the SAS data set that will contain detailed
          results (scale values, standard deviations, category
          boundaries, probits, etc.)

OUTDSN2 - the name of the SAS data set that will contain summary
          results (scale values and standard deviations)

******************************************************************;

proc iml worksize=60;
reset noprint nodetails noall noflow nolog;
start catb;
use &dsn;
read all var{ stimulus
%do k=1 %to &ncats;
   _%scan(&catlabs,&k)
%end;
} into indata;

close &dsn;
sname=rowcatc((j(nrow(indata),1,"s")||char(indata[,1],3)));
rname={
%do k=1 %to &ncats;
%if &k=&ncats %then %let sym=}%str(;);%else %let sym=,;
   %scan(&catlabs,&k)&sym
%end;
rname=rowcatc((j(&ncats,1,"r")||char(rname,3)));
response=indata[,2:%eval(&ncats+1)];
nobs=sum(response)/(nrow(sname));
free indata;
print / "************************************************************";
print   "************************************************************";
print   "*************** The Following Results Are For: *************";
print , , "Number Of Trials By Subjects" nobs;
print , , "Response Matrix", response[format=3.0 rowname=sname colname=rname] ;
prop=response;
cumprop=prop;
do j=1 to nrow(prop);
   prop[j,]=prop[j,]/sum(prop[j,]);
   cumprop[j,]=cusum(prop[j,]);
end;
print , , "Matrix Of Proportions", prop[format=4.2 rowname=sname colname=rname] ;
free response prop;
check=cumprop>=&lbp & cumprop<=&ubp;
cloc=loc(check[,<>]);
if ncol(cloc)^=nrow(cumprop) then do;
   aloc=loc(check[,<>]-j(nrow(cumprop),1,1));
   print "******************** NOTE ***********************";
   print , "The Following Stumuli Have Been Discarded Due To";
   print   "Lack Of Response Variablity", (sname[aloc,1]);
   sname=sname[cloc,1];
   cumprop=cumprop[cloc,];
end;
check=cumprop>=&lbp & cumprop<=&ubp;
cloc=loc(check[<>,]);
aloc=cloc[1,ncol(cloc)]+1;
cloc=cloc||aloc;
if ncol(cloc)^=ncol(cumprop) then do;
   aloc=loc(check[<>,]-j(1,nrow(cumprop),1));
   print "******************** NOTE ***********************";
   print , "The Following Categories Have Been Discarded";
   print   "Due To Lack Of Use", (rname[aloc,1]);
   rname=rname[cloc,1];
   cumprop=cumprop[,cloc];
end;
probits=j(nrow(cumprop),(ncol(cumprop)-1),.);
do j=1 to nrow(cumprop);
   do l=1 to (ncol(cumprop)-1);
      if ^(cumprop[j,l]>&ubp | cumprop[j,l]< &lbp) then
      probits[j,l]=probit(cumprop[j,l]);
   end;
end;
print , , "Matrix Of Cummulative Proportions", cumprop[format=5.3 rowname=sname colname=rname]
;
print , , "Matrix Of Probits (Z-scores)", probits[format=5.3 rowname=sname colname=rname] ;
free response prop;
mname={"X"}//sname;
if all(probits^=.) then do;
   ones=j(1,ncol(probits),1);
   scalestd=(probits-probits[,:]*ones)*(probits-probits[,:]*ones)`/ncol(probits);
   scalestd=diag(1/sqrt(abs(scalestd)));
   catbound=scalestd*probits;
   scalestd=scalestd[,+];
   catbound=catbound[:,];
   scale=(j(5,1,1)*catbound[,:])-(scalestd#probits[,:]);
   writit=scale||probits;
   writit=({.}||catbound)//writit;
   create &outdsn1 from writit [rowname=mname];
   append from writit [rowname=mname];
   close &outdsn1;
   print , , , "********* Complete Data Matrix Solution **********";
   flag=0;
end;
if ^all(probits^=.) then do;
   aprobits=probits[,2:(ncol(probits))]-probits[,1:(ncol(probits)-1)];
   do i=1 to ncol(aprobits);
      if any(aprobits[,i])=0 then goto stopit;
   end;
   cmat=ncol(aprobits)-1;
   cmat=j(1,cmat,.);
   do j=1 to ncol(cmat);
      count=0;
      aprobuse=j(nrow(aprobits),2,.);
      do i=1 to nrow(aprobits);
         if all(aprobits[i,j:(j+1)]^=.) then do;
            count=count+1;
            aprobuse[count,1:2]=aprobits[i,j:(j+1)];
         end;
      end;
      if count=0 then goto stopit;
      aprobuse=aprobuse[1:count,];
      cmat[1,j]=1/(aprobuse[,1]`*aprobuse[,1])*aprobuse[,1]`*aprobuse[,2];
   end;
   free aprobuse;
   catbound=j(1,ncol(probits),0);
   catbound[1,2]=1;
   do i=1 to ncol(cmat);
      catbound[1,(i+2)]=cmat[1,i]#(catbound[1,(i+1)]-catbound[1,i])+catbound[1,(i+1)];
   end;
   aprobuse=j(ncol(catbound),1,1)||catbound`||(probits`);
   scale=j((ncol(aprobuse)-2),1,.);
   scalestd=j((ncol(aprobuse)-2),1,.);
   do j=3 to ncol(aprobuse);
      count=0;
      x=j(nrow(aprobuse),2,.);
      y=j(nrow(aprobuse),1,.);
      do i=1 to nrow(aprobuse);
         if all(aprobuse[i,1:2]^=. & aprobuse[i,j]^=.) then do;
            count=count+1;
            x[count,]=aprobuse[i,1:2];
            y[count,]=aprobuse[i,j];
         end;
      end;
      if count=1 then goto stopit;
      x=x[1:count,];
      y=y[1:count,];
      cmat=inv(x`*x)*x`*y;
      scale[(j-2),1]=-cmat[1,1]/cmat[2,1];
      scalestd[(j-2),1]=1/cmat[2,1];
   end;
   writit=scale||probits;
   writit=({.}||catbound)//writit;
   create &outdsn1 from writit [rowname=mname];
   append from writit [rowname=mname];
   close &outdsn1;
   free probits;
   print , , , "Matrix Of Probit Differences" aprobits[format=5.2 rowname=sname
colname=rname];
end;
print , , , "Category Boundaries", catbound [format=7.3  colname=rname];
print , , , "Scale Values", scale [format=7.3 rowname=sname];
print , , , "Stimulus STDs", scalestd [format=7.3 rowname=sname];
free / rname sname scale scalestd catbound cumprop nobs flag;
cumprop=cumprop[,1:(ncol(cumprop)-1)];
aprobits=cumprop;
do i=1 to (nrow(scale));
    do j=1 to (ncol(catbound));
       aprobits[i,j]=1/scalestd[i,1]*(catbound[1,j]-scale[i,1]);
    end;
end;
aprobits=probnorm(aprobits);
print ,,,"Reproduced Cumulative Propotions",aprobits[format=5.3 rowname=sname colname=rname];
thurst=abs(cumprop-aprobits);
print ,,,"Difference Between Reproduced And Original Proportions",thurst[format=5.3
rowname=sname colname=rname];
thurst=thurst[,:];
   writit=scale||scalestd||thurst;
   create &outdsn2 from writit [rowname=sname];
   append from writit [rowname=sname];
   close &outdsn2;
print ,,,"Difference Between Reproduced And Original Proportions";
print  "Averaged Across Categories",thurst[format=5.3 rowname=sname colname=rname];
thurst=thurst[:];
print ,,,"Average Absolute Discrepancy Between Observed";
print "And Predicted Proportions (Edwards and Thurstone, 1952)";
print , thurst;
aprobits=57.296*(arsin(sqrt(cumprop))-arsin(sqrt(aprobits)));
free cumprop;
mostell=trace(aprobits*aprobits`)*nobs/821;
df=sum((aprobits^=.))-(ncol(catbound)-2+nrow(scale)+nrow(scalestd));
free aprobits;
p=1-probchi(mostell,df);
print ,,,"Mosteller''s Chi-Square Test (Guilford, 1954)";
print "Note: This Test May Not Be Appropriate (See Torgerson,1958)";
print ,, "Chi-Square ", mostell,"P-value" p[format=7.4],"Degrees of Freedom" df;
goto allok;
stopit:
print , , , "Incomplete Data Matrix Is Too Sparse To Scale.";
print , "********* No Solution For This Block **********";
allok:
free /;
finish;
run catb;
quit;
%mend;
%catjug(intoit,.05,.95,9,1 2 3 4 5 6 7 8 9,scalev1,scalev2);

proc print data=scalev1;
run;
proc print data=scalev2;
run;
data tscale;
set scalev2;
rename col1=SCALEV col2=STD;
keep sname col1 col2;
run;
PROC SORT; BY SCALEV;
proc print; run;

